# Getestet mit Python 3.11.6 und PySide 6.5.3

import math
import PySide6.QtWidgets as widgets
import PySide6.QtCore as core
import PySide6.QtGui as gui

class MeinWidget(widgets.QWidget):
    def __init__(self, parent : widgets.QWidget|None = None):
        super().__init__(parent)
        self.setCursor(core.Qt.CursorShape.BlankCursor)
        self.setAttribute(core.Qt.WidgetAttribute.WA_AcceptTouchEvents)
        self._scale : float = 1
        self._angle : float = 0 # Bogenmaß
        self._shiftX : float = 0
        self._shiftY : float = 0
        self._img = gui.QImage("test.png")
        self._points : list[core.QPointF] = []
        self._br = gui.QBrush(core.Qt.GlobalColor.black)
        self._br.setStyle(core.Qt.BrushStyle.SolidPattern)

    def paintEvent(self, event: gui.QPaintEvent) -> None:
        super().paintEvent(event)
        painter = gui.QPainter(self)
        painter.translate(self._shiftX, self._shiftY)
        painter.rotate(self._angle * 180.0 / math.pi)
        painter.scale(self._scale, self._scale)
        painter.drawImage(core.QPoint(), self._img)
        painter.resetTransform()
        painter.setBrush(self._br)
        for q in self._points:
            painter.drawEllipse(q, 20.0, 20.0)
        painter.end()

    def event(self, event: core.QEvent) -> bool:
        if isinstance(event, gui.QTouchEvent):
            tp = event.points()
            self._points.clear()
            if event.type() != core.QEvent.Type.TouchEnd:
                for p in tp:
                    self._points.append(p.pos())
            if len(tp) == 2:
                pOneNew = tp[0].pos()
                pTwoNew = tp[1].pos()
                pOneOld = tp[0].lastPos()
                pTwoOld = tp[1].lastPos()
                
                diffNew = pOneNew - pTwoNew
                diffOld = pOneOld - pTwoOld
                scaleD = math.sqrt((diffNew.x()**2+diffNew.y()**2)/(diffOld.x()**2+diffOld.y()**2))
                self._scale *= scaleD
                angleD = math.atan2(diffNew.y(),diffNew.x())-math.atan2(diffOld.y(),diffOld.x())
                self._angle += angleD
                self._shiftX = pOneNew.x() - scaleD*(math.cos(angleD)*(pOneOld.x()-self._shiftX)-math.sin(angleD)*(pOneOld.y()-self._shiftY))
                self._shiftY = pOneNew.y() - scaleD*(math.sin(angleD)*(pOneOld.x()-self._shiftX)+math.cos(angleD)*(pOneOld.y()-self._shiftY))
            self.update()
            return True
        else:
            return super().event(event)